import {
  addDependenciesToPackageJson,
  formatFiles,
  generateFiles,
  joinPathFragments,
  names,
  OverwriteStrategy,
  readJson,
  readNxJson,
  readProjectConfiguration,
  Tree,
  updateJson,
} from '@nx/devkit';
import { LibGeneratorSchema } from './schema';

import { libraryGenerator } from '@nx/js';

export async function libGenerator(tree: Tree, options: LibGeneratorSchema) {
  const projectConfig = readProjectConfiguration(tree, options.project);

  const typeDirectory = (() => {
    switch (options.type) {
      case 'module':
        return 'modules';
      case 'feature':
        return 'features';
    }
  })();

  const libraryName = `${projectConfig.name}-${options.name}`;
  const libraryRoot = joinPathFragments(
    projectConfig.root,
    'src',
    typeDirectory,
    libraryName
  );

  const opts: Parameters<typeof libraryGenerator>[1] = {
    directory: libraryRoot,
    name: libraryName,
    buildable: true,
    linter: 'eslint',
    setParserOptionsProject: true,
    unitTestRunner: 'jest',
    strict: true,
    simpleName: false,
  };
  await libraryGenerator(tree, opts);

  tree.delete(joinPathFragments(libraryRoot, 'src'));
  convertToCommonJs(tree, libraryRoot);
  generateFiles(
    tree,
    joinPathFragments(__dirname, './files/common'),
    libraryRoot,
    { ...opts, ...names(options.name), controller: true, service: true },
    { overwriteStrategy: OverwriteStrategy.Overwrite }
  );
  await formatFiles(tree);

  addDependenciesToPackageJson(
    tree,
    {},
    { [`@${getNpmScope(tree)}/${libraryName}`]: 'workspace:' },
    joinPathFragments(projectConfig.root, 'package.json')
  );
}

function convertToCommonJs(tree: Tree, libraryRoot: string) {
  updateJson(
    tree,
    joinPathFragments(libraryRoot, 'package.json'),
    (pkgJson) => {
      pkgJson.type = 'commonjs';
      return pkgJson;
    }
  );
}

function getNpmScope(tree: Tree): string | undefined {
  const { name } = tree.exists('package.json')
    ? readJson<{ name?: string }>(tree, 'package.json')
    : { name: null };

  return name?.startsWith('@') ? name.split('/')[0].substring(1) : undefined;
}

export default libGenerator;
