import {
  FileChange,
  formatFiles,
  generateFiles,
  joinPathFragments,
  OverwriteStrategy,
  readProjectConfiguration,
  Tree,
  workspaceRoot,
} from '@nx/devkit';
import execa from 'execa';
import os from 'node:os';
import { mkdirSync, writeFileSync, chmodSync, rmSync } from 'node:fs';
import { mkdtemp, rm } from 'node:fs/promises';
import { dirname, join } from 'node:path';

import { TomgGeneratorSchema } from './schema';

export async function tomgGenerator(tree: Tree, options: TomgGeneratorSchema) {
  const libraryRoot = readProjectConfiguration(tree, options.project).root;

  generateFiles(
    tree,
    joinPathFragments(__dirname, './files'),
    libraryRoot,
    options,
    { overwriteStrategy: OverwriteStrategy.KeepExisting }
  );
  flushChanges(workspaceRoot, tree.listChanges());

  const tmpDirPath = await mkdtemp(join(os.tmpdir(), 'tomg-'));
  const { stdout, stderr } = await execa(
    `typeorm-model-generator`,
    [
      '-o',
      tmpDirPath,
      '-e',
      options.driver,
      '-h',
      options.host,
      '-d',
      options.database,
      '-u',
      options.username,
      '-x',
      options.password,
    ],
    { cwd: libraryRoot }
  );
  if (!options.quiet) {
    console.log(stdout);
    console.error(stderr);
  }

  generateFiles(
    tree,
    tmpDirPath,
    joinPathFragments(libraryRoot, options.directory),
    options,
    { overwriteStrategy: OverwriteStrategy.Overwrite }
  );

  await formatFiles(tree);
  await rm(tmpDirPath, { recursive: true, force: true });

  console.log();
  console.log('Generation complete.');
  console.log();
}

export default tomgGenerator;

function flushChanges(root: string, fileChanges: FileChange[]): void {
  fileChanges.forEach((f) => {
    const fpath = join(root, f.path);
    if (f.type === 'CREATE') {
      mkdirSync(dirname(fpath), { recursive: true });
      writeFileSync(fpath, f.content!);
      if (f.options?.mode) chmodSync(fpath, f.options.mode);
    } else if (f.type === 'UPDATE') {
      writeFileSync(fpath, f.content!);
      if (f.options?.mode) chmodSync(fpath, f.options.mode);
    } else if (f.type === 'DELETE') {
      rmSync(fpath, { recursive: true, force: true });
    }
  });
}
